/**
 * Run the autoplay patch:
 * 1. Mark current search results as ignorable.
 * 2. Wait for new search results
 * 3. Simulate "click" on first watchable tile in new search results.
 */
(function autoplayPatch(window, document, searchText) {
    const CHECK_INTERVAL = 100; // Somewhat arbitrary interval to wake up and check for new search results
    const RETRY_TIMEOUT = 15000; // Overall time before we give up waiting for new search results
    const PAUSE_BEFORE_CLICK = 25; // Delay allows new search results to be visible before clicking
    const SEARCH_RESULTS_IGNORE_CLASS_NAME = 'amzn-autoplay-ignore';
    const SEARCH_RESULTS_IGNORE_SELECTOR = '#search-results.' + SEARCH_RESULTS_IGNORE_CLASS_NAME;
    const EXISTING_SEARCH_RESULTS_SELECTOR = '#search-results:not(.' + SEARCH_RESULTS_IGNORE_CLASS_NAME + ')';
    const WATCHABLE_TILE_SELECTOR = '#search-results .watchable-tile:not(.airstream-playlist-tile)';
    const SEARCH_TEXT_SELECTOR = '#text-input .emulated-input-text';

     /**
      * Helper function for simulating click on an HTML element
      */
     const clickElement = function(element) {
        // NOTE: Use of keydown event is unreliable, stops working for unknown reason
        // element.dispatchEvent(new KeyboardEvent('keydown', {'keyCode':13}));

        // Simulating mousedown, then mouseup works reliably
        element.dispatchEvent(new MouseEvent('mousedown'));
        element.dispatchEvent(new MouseEvent('mouseup'));
     };

    /**
     * Clicks the first watchable result from an in-progress or recently finished search.
     */
    const clickFirstSearchResult = function() {
        // If there are still ignored search results, don't play anything yet
        if (document.querySelector(SEARCH_RESULTS_IGNORE_SELECTOR)) {
            return;
        }

        // In some cases, the originally search results are updated before the new
        // results are shown, resulting in an autoplay on the old search results.
        // By checking the searchText, we are able to validate we have the right search results
        if (searchText != "" && searchText != document.querySelector(SEARCH_TEXT_SELECTOR).textContent) {
            return;
        }

        // Find first watchable tile in new search results
        const tile = document.querySelector(WATCHABLE_TILE_SELECTOR);

        // No tile yet, return
        if (!tile) {
            return;
        }

        // Clear interval and timeout
        clearInterval(window.autoplayPatch.intervalId);
        clearTimeout(window.autoplayPatch.timeoutId);

        // Click after a small delay so customer sees search results
        // (otherwise looks like we are clicking previous search results)
        setTimeout(function clickInAMoment() {
            // tile.dispatchEvent(new KeyboardEvent('keydown', {'keyCode':13}))
            clickElement(tile);
        }, PAUSE_BEFORE_CLICK);
    };

    // Initialize and attach to window object if this is the first run
    // We do this so we can clear any previous pending autoplay attempts
    if (window.autoplayPatch === undefined) {
        window.autoplayPatch = {
            timeoutId: null,
            intervalId: null
        };
    } else {
        // Safely clear any previous autoplay interval or timeout
        clearInterval(window.autoplayPatch.intervalId);
        clearTimeout(window.autoplayPatch.timeoutId);
    }

    // Mark existing/previous search results as ignorable
    const searchResultsEl = document.querySelector(EXISTING_SEARCH_RESULTS_SELECTOR);
    if (searchResultsEl) {
        searchResultsEl.classList.add(SEARCH_RESULTS_IGNORE_CLASS_NAME);
    }

    // Start interval to check for new search results
    window.autoplayPatch.intervalId = setInterval(clickFirstSearchResult, CHECK_INTERVAL);

    // If we have not autoplayed in AUTOPLAY_TIMEOUT ms, then stop trying
    window.autoplayPatch.timeoutId = setTimeout(() => {
        clearInterval(window.autoplayPatch.intervalId);
    }, RETRY_TIMEOUT);

}(window, document, "%s"));
